use anyhow::{bail, ensure, Context, Result};
use clap::Parser;
use itertools::Itertools;
use serde_json::Value;

mod cli;

#[derive(Debug)]
pub(crate) struct ReinstallConfig {
    pub(crate) bootc_image: String,
}

impl ReinstallConfig {
    pub fn parse_from_cli(cli: cli::Cli) -> Result<Self> {
        Ok(Self {
            bootc_image: cli.bootc_image,
        })
    }

    pub fn parse_from_config_file(config_bytes: &[u8]) -> Result<Self> {
        let value: Value = serde_yaml::from_slice(config_bytes)?;

        let mut value = value
            .as_object()
            .context("config file must be a YAML object")?
            .clone();

        let bootc_image = match value.remove("bootc_image") {
            Some(value) => value
                .as_str()
                .context("bootc_image must be a string")?
                .to_string(),
            None => bail!("bootc_image is required"),
        };

        ensure!(
            value.is_empty(),
            "unknown keys {:?} in config file",
            value.keys().map(|key| key.to_string()).join(", ")
        );

        let reinstall_config = Self { bootc_image };

        Ok(reinstall_config)
    }

    pub fn load() -> Result<Self> {
        Ok(match std::env::var("BOOTC_REINSTALL_CONFIG") {
            Ok(var) => {
                ensure_no_cli_args()?;

                ReinstallConfig::parse_from_config_file(
                    &std::fs::read(&var)
                        .context(format!("reading BOOTC_REINSTALL_CONFIG file {}", var))?,
                )
                .context(format!("parsing BOOTC_REINSTALL_CONFIG file {}", var))?
            }
            Err(_) => ReinstallConfig::parse_from_cli(cli::Cli::parse()).context("CLI parsing")?,
        })
    }
}

fn ensure_no_cli_args() -> Result<()> {
    let num_args = std::env::args().len();

    ensure!(
        num_args == 1,
        "BOOTC_REINSTALL_CONFIG is set, but there are {num_args} CLI arguments. BOOTC_REINSTALL_CONFIG is meant to be used with no arguments."
    );

    Ok(())
}
