use super::ROOT_KEY_MOUNT_POINT;
use crate::users::UserKeys;
use anyhow::Context;
use std::process::Command;

pub(crate) fn command(image: &str, root_key: Option<UserKeys>) -> Vec<String> {
    let mut podman_command_and_args = vec![
        // We use podman to run the bootc container. This might change in the future to remove the
        // podman dependency.
        "podman",
        "run",
        // The container needs to be privileged, as it heavily modifies the host
        "--privileged",
        // The container needs to access the host's PID namespace to mount host directories
        "--pid=host",
        // Since https://github.com/containers/bootc/pull/919 this mount should not be needed, but
        // some reason with e.g. quay.io/fedora/fedora-bootc:41 it is still needed.
        "-v",
        "/var/lib/containers:/var/lib/containers",
    ];

    let mut bootc_command_and_args = vec![
        "bootc",
        "install",
        // We're replacing the current root
        "to-existing-root",
        // The user already knows they're reinstalling their machine, that's the entire purpose of
        // this binary. Since this is no longer an "arcane" bootc command, we can safely avoid this
        // timed warning prompt. TODO: Discuss in https://github.com/containers/bootc/discussions/1060
        "--acknowledge-destructive",
    ];

    if let Some(root_key) = root_key.as_ref() {
        podman_command_and_args.push("-v");
        podman_command_and_args.push(&root_key.authorized_keys_path);
        podman_command_and_args.push(ROOT_KEY_MOUNT_POINT);

        bootc_command_and_args.push("--root-ssh-authorized-keys");
        bootc_command_and_args.push(&ROOT_KEY_MOUNT_POINT);
    }

    podman_command_and_args
        .iter()
        .chain([image].iter())
        .chain(bootc_command_and_args.iter())
        .map(|x| x.to_string())
        .collect::<Vec<String>>()
}

pub(crate) fn run_podman(all_args: Vec<String>) -> Result<(), anyhow::Error> {
    Command::new(all_args[0].clone())
        .args(all_args[1..].iter())
        .status()
        .context(format!(
            "Failed to run the command \"{}\"",
            all_args.join(" ")
        ))?;
    Ok(())
}
