use anyhow::{Context, Result};
use std::fmt::Display;
use std::fmt::Formatter;
use uzers::os::unix::UserExt;

#[derive(Clone, Debug)]
pub(crate) struct UserKeys {
    pub(crate) user: String,
    pub(crate) authorized_keys: String,
    pub(crate) authorized_keys_path: String,
}

impl UserKeys {
    pub(crate) fn num_keys(&self) -> usize {
        self.authorized_keys.lines().count()
    }
}

impl Display for UserKeys {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "User {} ({} authorized keys)",
            self.user,
            self.num_keys()
        )
    }
}

/// # Safety
/// See safety information in [`uzers::all_users`]. We are not impacted because we're not
/// multi-threaded.
pub(crate) fn get_all_users_keys() -> Result<Vec<UserKeys>> {
    #[allow(unsafe_code)]
    let iter = unsafe { uzers::all_users() };

    let mut result = Vec::new();

    for user_info in iter {
        if user_info.uid() < 1000 && user_info.uid() != 0 {
            tracing::debug!(
                "Skipping user {} with uid {} because it's a system user (uid < 1000)",
                user_info.name().to_string_lossy(),
                user_info.uid()
            );
            continue;
        }

        if ["/bin/false", "/bin/true", "nologin"]
            .into_iter()
            .any(|x| user_info.shell().to_string_lossy().contains(x))
        {
            tracing::debug!(
                "Skipping user {} with shell {} because it can't login",
                user_info.name().to_string_lossy(),
                user_info.shell().to_string_lossy()
            );
            continue;
        }

        let home_dir = user_info.home_dir();
        let user_authorized_keys_path = home_dir.join(".ssh/authorized_keys");
        if !user_authorized_keys_path.exists() {
            tracing::debug!(
                "Skipping user {} because it doesn't have an SSH authorized_keys file",
                user_info.name().to_string_lossy()
            );
            continue;
        }

        let user_name = user_info
            .name()
            .to_str()
            .context("user name is not valid utf-8")?;

        let user_authorized_keys = std::fs::read_to_string(&user_authorized_keys_path)
            .context("Failed to read user's authorized keys")?;

        if user_authorized_keys.trim().is_empty() {
            tracing::debug!(
                "Skipping user {} because it has an empty SSH authorized_keys file",
                user_info.name().to_string_lossy()
            );
            continue;
        }

        let user_keys = UserKeys {
            user: user_name.to_string(),
            authorized_keys: user_authorized_keys,
            authorized_keys_path: user_authorized_keys_path
                .to_str()
                .context("user's authorized_keys path is not valid utf-8")?
                .to_string(),
        };

        tracing::debug!(
            "Found user {} with {} SSH authorized_keys",
            user_keys.user,
            user_keys.num_keys()
        );

        result.push(user_keys);
    }

    Ok(result)
}
